from typing import (
    ClassVar,
    List,
    TYPE_CHECKING,
    Type,
)

from .base import (
    Check,
    CheckInvoker,
    MissingPropertyCheck,
    ObjectIsNotNoneCheck,
)

if TYPE_CHECKING:
    from ..models.credential import Credential  # noqa: F401


class CredentialCheck(CheckInvoker):
    checkers: ClassVar[List[Type[Check["Credential"]]]] = []
    object_name = "credential"


@CredentialCheck.checker
class UC01(ObjectIsNotNoneCheck["Credential"]):
    ...


@CredentialCheck.property_checker(property_name="username")
class UC02(MissingPropertyCheck["Credential"]):
    ...


@CredentialCheck.property_checker(property_name="identity_id")
class UC03(MissingPropertyCheck["Credential"]):
    ...


@CredentialCheck.property_checker(property_name="identity_username")
class UC04(MissingPropertyCheck["Credential"]):
    ...
