from typing import (
    ClassVar,
    List,
    Optional,
    TYPE_CHECKING,
    Type,
)

from .base import (
    Check,
    CheckInvoker,
    MissingPropertyCheck,
    ObjectIsNotNoneCheck,
    PropertyConstraintCheck,
)

if TYPE_CHECKING:
    from ..models.mapped_collection import MappedCollection  # noqa: F401


class MappedCollectionCheck(CheckInvoker):
    checkers: ClassVar[List[Type[Check["MappedCollection"]]]] = []
    object_name = "mapped_collection"


@MappedCollectionCheck.checker
class MC01(ObjectIsNotNoneCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="display_name")
class MC02(MissingPropertyCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="force_verify")
class MC03(MissingPropertyCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="disable_verify")
class MC04(MissingPropertyCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="enable_https")
class MC05(MissingPropertyCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="allow_guest_collections")
class MC06(MissingPropertyCheck["MappedCollection"]):
    ...


@MappedCollectionCheck.property_checker(property_name="sharing_users_allow")
class MC07(PropertyConstraintCheck["MappedCollection"]):
    constraint_name = "sharing disabled but sharing_users_allow is set"

    def value_is_ok(self, obj: Optional["MappedCollection"]) -> bool:
        ok = True

        if obj and obj.allow_guest_collections is False and obj.sharing_users_allow:
            ok = False

        return ok


@MappedCollectionCheck.property_checker(property_name="sharing_users_deny")
class MC08(PropertyConstraintCheck["MappedCollection"]):
    constraint_name = "sharing disabled but sharing_users_deny is set"

    def value_is_ok(self, obj: Optional["MappedCollection"]) -> bool:
        ok = True

        if obj and obj.allow_guest_collections is False and obj.sharing_users_deny:
            ok = False

        return ok


@MappedCollectionCheck.property_checker(property_name="sharing_groups_allow")
class MC09(PropertyConstraintCheck["MappedCollection"]):
    constraint_name = "sharing disabled but sharing_groups_allow is set"

    def value_is_ok(self, obj: Optional["MappedCollection"]) -> bool:
        ok = True

        if obj and obj.allow_guest_collections is False and obj.sharing_groups_allow:
            ok = False

        return ok


@MappedCollectionCheck.property_checker(property_name="sharing_groups_deny")
class MC10(PropertyConstraintCheck["MappedCollection"]):
    constraint_name = "sharing disabled but sharing_groups_deny is set"

    def value_is_ok(self, obj: Optional["MappedCollection"]) -> bool:
        ok = True

        if obj and obj.allow_guest_collections is False and obj.sharing_groups_deny:
            ok = False

        return ok
