from typing import Optional

import click

from ..models import MigrationPlan, GlobusOIDCIdP, GlobusOIDCIdPDict


@click.command("add-globus-oidc")
@click.option(
    "--display-name",
    required=True,
    metavar="NAME",
    help="Display name of this OIDC server",
)
@click.option(
    "--support-email",
    required=True,
    metavar="EMAIL",
    help="Email address of the support person for this server",
)
@click.option(
    "--support-contact",
    required=True,
    metavar="CONTACT",
    help="Name of the support person for this server",
)
@click.option(
    "--server-name",
    required=True,
    metavar="DOMAIN",
    help="Fully qualified domain name of the identity server",
)
@click.option(
    "--certificate-path",
    required=True,
    metavar="CERT",
    help="Path to the certificate that matches the server name",
)
@click.option(
    "--certificate-chain-path",
    required=True,
    metavar="CHAIN",
    help="Path to the certificate chain containing any intermediate "
    "certificates required to validate the host certificate",
)
@click.option(
    "--private-key-path",
    required=True,
    metavar="KEY",
    help="Path to the private key used by the server",
)
@click.option(
    "--manage-certificate-and-key",
    is_flag=True,
    default=False,
    help="Flag to enable GCS managing the distribution of the certificate and key between DTNs",
)
def add_globus_oidc(
    display_name: Optional[str],
    support_email: Optional[str],
    support_contact: Optional[str],
    server_name: Optional[str],
    certificate_path: Optional[str],
    certificate_chain_path: Optional[str],
    private_key_path: Optional[str],
    manage_certificate_and_key: bool,
) -> None:
    migration_plan = MigrationPlan.load()

    oidc_server_dict: GlobusOIDCIdPDict = dict(
        display_name=display_name,
        support_email=support_email,
        support_contact=support_contact,
        server_name=server_name,
        certificate_path=certificate_path,
        certificate_chain_path=certificate_chain_path,
        private_key_path=private_key_path,
        manage_certificate_and_key=manage_certificate_and_key,
    )
    migration_plan.oidc_server = GlobusOIDCIdP(data=oidc_server_dict)
    migration_plan.dump_oidc_server()


if __name__ == "__main__":
    add_globus_oidc()
