import click

from ..models import MigrationPlan


@click.command("add-idps-to-storage-gateway")
def add_idps_to_storage_gateway() -> None:
    migration_plan = MigrationPlan.load()
    if not migration_plan.storage_gateway:
        raise click.ClickException("No storage gateway object to add idps to")

    domains = set()
    identity_mappings = []

    if migration_plan.guest_collections:
        for gc in migration_plan.guest_collections:
            if gc.credential and gc.credential.mapping:
                identity_mappings.append(gc.credential.mapping)

    if migration_plan.identity_providers:
        for idp in migration_plan.identity_providers.values():
            if idp.domains:
                for domain in idp.domains:
                    domains.add(domain)
            if idp.mapping:
                identity_mappings.append(idp.mapping)

    migration_plan.storage_gateway.allowed_domains = list(domains)
    migration_plan.storage_gateway.identity_mappings = identity_mappings

    migration_plan.dump_storage_gateway()
