from typing import TYPE_CHECKING
import click

from ..models import MigrationPlan

if TYPE_CHECKING:
    import uuid


@click.command(
    "edit-credential",
    help="Edit identity associated with a guest collection credential",
)
@click.argument("GUEST_COLLECTION_ID", type=click.UUID)
@click.argument("GLOBUS_USERNAME")
def edit_credential(guest_collection_id: "uuid.UUID", globus_username: str) -> None:
    migration_plan = MigrationPlan.load()

    # Look up domain of globus_username in the configured idp list and the oidc
    # config if present
    user, domain = globus_username.rsplit("@", 1)

    idp = migration_plan.get_identity_provider_for_domain(domain)
    if not idp:
        oidc_server = migration_plan.oidc_server
        if not oidc_server or domain != oidc_server.server_name:
            raise click.ClickException(f"No ID provider configured for {domain}")

    guest_collection = migration_plan.get_guest_collection_by_id(guest_collection_id)
    if not guest_collection:
        raise click.ClickException(f"Unknown guest collection {guest_collection_id}")

    user_credential = guest_collection.credential
    if user_credential:
        user_credential.identity_username = globus_username
        migration_plan.dump_guest_collection(str(guest_collection_id))
