from typing import Optional, Tuple

import click

from ..models import MigrationPlan, HostEndpoint


@click.command("edit-endpoint", help="Edit metadata about an endpoint")
@click.option("--contact-info", metavar="INFO", help="Contact point for this endpoint")
@click.option(
    "--department", metavar="DEPARTMENT", help="Department operating this endpoint"
)
@click.option(
    "--description", metavar="DESCRIPTION", help="Description of this endpoint"
)
@click.option(
    "--info-link", metavar="INFO", help="Link to a page with info about this endpoint"
)
@click.option(
    "--keyword",
    multiple=True,
    metavar="KEYWORD",
    help="Keywords to associate with this endpoint",
)
@click.option(
    "--organization",
    metavar="ORGANIZATION",
    help="Organization which operates this endpoint",
)
def edit_endpoint(
    contact_info: Optional[str],
    department: Optional[str],
    description: Optional[str],
    info_link: Optional[str],
    keyword: Tuple[str],
    organization: Optional[str],
) -> None:
    migration_plan = MigrationPlan.load()

    if not migration_plan.host_endpoint:
        migration_plan.set_host_endpoint(HostEndpoint())

    assert migration_plan.host_endpoint is not None

    if contact_info:
        migration_plan.host_endpoint.contact_info = contact_info
    if department:
        migration_plan.host_endpoint.department = department
    if description:
        migration_plan.host_endpoint.description = description
    if info_link:
        migration_plan.host_endpoint.info_link = info_link
    if keyword != tuple([]):
        migration_plan.host_endpoint.keywords = list(keyword)
    if organization:
        migration_plan.host_endpoint.organization = organization

    migration_plan.dump_host_endpoint()
