from typing import Optional
from typing_extensions import TypedDict

from ..typing.transfer_documents import (
    ACLPermissionName,
    AccessDocumentDict,
    PrincipalTypeName,
)
from ..util import yaml, yaml_object


class ACLDict(TypedDict):
    id: Optional[int]
    principal: Optional[str]
    principal_type: Optional[PrincipalTypeName]
    path: Optional[str]
    permissions: Optional[ACLPermissionName]
    create_time: Optional[str]


@yaml_object(yaml)
class ACL:
    def __init__(self) -> None:
        self.id: Optional[int] = None
        self.principal: Optional[str] = None
        self.principal_type: Optional[PrincipalTypeName] = None
        self.path: Optional[str] = None
        self.permissions: Optional[ACLPermissionName] = None
        self.create_time = None
        self.access_document = None

    @property
    def access_document(self) -> Optional[AccessDocumentDict]:
        return self._access_document

    @access_document.setter
    def access_document(self, value: Optional[AccessDocumentDict]) -> None:
        self._access_document = value

        if value:
            if self.id is None:
                self.id = value["id"]
            if self.principal is None:
                self.principal = value["principal"]
            if self.principal_type is None:
                self.principal_type = value["principal_type"]
            if self.path is None:
                self.path = value["path"]
            if self.permissions is None:
                self.permissions = value["permissions"]
            if self.create_time is None:
                self.create_time = value["create_time"]

    @staticmethod
    def from_access_document(access_document: AccessDocumentDict) -> "ACL":
        acl = ACL()
        acl.access_document = access_document
        return acl

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ACL):
            return False

        return self is other or self.__getstate__() == other.__getstate__()

    def __getstate__(self) -> ACLDict:
        return {
            "id": self.id,
            "principal": self.principal,
            "principal_type": self.principal_type,
            "path": self.path,
            "permissions": self.permissions,
            "create_time": self.create_time,
        }

    def __setstate__(self, state: ACLDict) -> None:
        self.id = state["id"]
        self.principal = state["principal"]
        self.principal_type = state["principal_type"]
        self.path = state["path"]
        self.permissions = state["permissions"]
        self.create_time = state["create_time"]
