from typing import Any
import os
from typing import Optional, Union

import globus_sdk


CLIENT_ID = {
    "default": "974dc87d-3261-4c7f-8e93-5f298fe7f43e",
    "sandbox": "48ddc584-3bcb-431e-94ca-2967643c90ef",
    "test": "0a692758-e147-4d19-8f7d-af5623874d17",
    "preview": "c9585fa7-31bc-4b7d-8155-7f8868d1d9e9",
    "integration": "30e01f99-6943-496c-9b4a-42c17d813edb",
    "staging": "cbb9e120-ac12-41ee-b966-dcd454cc0596",
}

__tokens = {}  # type: Any


def get_api(
    service: Optional[str] = None,
) -> Union[globus_sdk.TransferClient, globus_sdk.AuthClient]:
    global __tokens

    if not __tokens:
        environment = os.environ.get("GLOBUS_SDK_ENVIRONMENT")
        if not environment:
            environment = "default"
        client_id = CLIENT_ID[environment]

        client = globus_sdk.NativeAppAuthClient(client_id)
        client.oauth2_start_flow()
        authorize_url = client.oauth2_get_authorize_url(
            additional_params={
                "prompt": "login",
                "session_required_single_domain": "globusid.org",
            }
        )
        print(f"Please go to this URL and login: {authorize_url}")
        auth_code = input("Please enter the code you get after login here: ").strip()
        token_response = client.oauth2_exchange_code_for_tokens(auth_code)
        globus_auth_data = token_response.by_resource_server["auth.globus.org"]
        globus_transfer_data = token_response.by_resource_server[
            "transfer.api.globus.org"
        ]
        __tokens["auth"] = globus_auth_data["access_token"]
        __tokens["transfer"] = globus_transfer_data["access_token"]

    if service is None:
        service = "transfer"

    authorizer = globus_sdk.AccessTokenAuthorizer(__tokens[service])

    if service == "transfer":
        return globus_sdk.TransferClient(authorizer=authorizer)
    elif service == "auth":
        return globus_sdk.AuthClient(authorizer=authorizer)
    raise Exception("Unsupported service")
