from gcs_migration.typing.transfer_documents import AccessDocumentDict
from gcs_migration.models import ACL

from gcs_migration.check.acl import (
    ACLCheck,
    AC01,
    AC02,
    AC03,
    AC04,
    AC05,
    AC06,
    AC07,
)


def test_check_ac01(access_acl_document: AccessDocumentDict) -> None:
    ac01 = AC01()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.principal = None

    assert ac01.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC01), res)) == []

    assert not ac01.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC01), res)) != []

    acl.principal = "foo"
    assert ac01.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC01), res)) == []


def test_check_ac02(access_acl_document: AccessDocumentDict) -> None:
    ac02 = AC02()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.principal = None

    assert ac02.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC02), res)) == []

    assert ac02.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC02), res)) == []

    acl.principal = "foo"
    assert not ac02.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC02), res)) != []

    acl.principal = "30907d78-d009-46b7-9971-bc9de259d953"
    assert ac02.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC02), res)) == []


def test_check_ac03(access_acl_document: AccessDocumentDict) -> None:
    ac03 = AC03()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.principal_type = None

    assert ac03.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC03), res)) == []

    assert not ac03.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC03), res)) != []

    acl.principal_type = "identity"
    assert ac03.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC03), res)) == []


def test_check_ac04(access_acl_document: AccessDocumentDict) -> None:
    ac04 = AC04()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.principal_type = "identity"

    assert ac04.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC04), res)) == []

    assert ac04.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC04), res)) == []

    acl.principal_type = "group"
    assert ac04.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC04), res)) == []

    acl.principal_type = "GROUP"  # type: ignore
    assert not ac04.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC04), res)) != []


def test_check_ac05(access_acl_document: AccessDocumentDict) -> None:
    ac05 = AC05()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.path = None

    assert ac05.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC05), res)) == []

    assert not ac05.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC05), res)) != []

    acl.path = "group"
    assert ac05.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC05), res)) == []


def test_check_ac06(access_acl_document: AccessDocumentDict) -> None:
    ac06 = AC06()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.permissions = None

    assert ac06.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC06), res)) == []

    assert not ac06.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC06), res)) != []

    acl.permissions = "r"
    assert ac06.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC06), res)) == []

    acl.permissions = "rw"
    assert ac06.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC06), res)) == []

    acl.permissions = "READ"  # type: ignore
    assert ac06.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC06), res)) == []


def test_check_ac07(access_acl_document: AccessDocumentDict) -> None:
    ac07 = AC07()
    checker = ACLCheck()
    acl = ACL()
    acl.access_document = access_acl_document
    acl.permissions = "r"

    assert ac07.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, AC07), res)) == []

    assert ac07.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC07), res)) == []

    acl.permissions = "rw"
    assert ac07.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC07), res)) == []

    acl.permissions = "READ"  # type: ignore
    assert not ac07.is_ok(acl)
    res = checker.check(acl)
    assert list(filter(lambda x: isinstance(x, AC07), res)) != []
