from gcs_migration.models import Credential, CredentialDict

from gcs_migration.check.credential import (
    CredentialCheck,
    UC01,
    UC02,
    UC03,
    UC04,
)


def test_credential_uc01(credential: CredentialDict) -> None:
    uc01 = UC01()

    assert not uc01.is_ok(None)

    cred = Credential(credential)
    assert uc01.is_ok(cred)

    checker = CredentialCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, UC01), res)) != []

    res = checker.check(cred)
    assert list(filter(lambda x: isinstance(x, UC01), res)) == []


def test_credential_uc02(credential: CredentialDict) -> None:
    uc02 = UC02()

    assert uc02.is_ok(None)

    cred = Credential(credential)
    cred.username = None
    assert not uc02.is_ok(cred)

    checker = CredentialCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, UC02), res)) == []

    res = checker.check(cred)
    assert list(filter(lambda x: isinstance(x, UC02), res)) != []


def test_credential_uc03(credential: CredentialDict) -> None:
    uc03 = UC03()

    assert uc03.is_ok(None)

    cred = Credential(credential)
    cred.identity_id = None
    assert not uc03.is_ok(cred)

    checker = CredentialCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, UC03), res)) == []

    res = checker.check(cred)
    assert list(filter(lambda x: isinstance(x, UC03), res)) != []


def test_credential_uc04(credential: CredentialDict) -> None:
    uc04 = UC04()

    assert uc04.is_ok(None)

    cred = Credential(credential)
    cred.identity_username = None
    assert not uc04.is_ok(cred)

    checker = CredentialCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, UC04), res)) == []

    res = checker.check(cred)
    assert list(filter(lambda x: isinstance(x, UC04), res)) != []
