import os
import pathlib

from gcs_migration.models import GlobusOIDCIdP

from gcs_migration.check.globus_oidc_idp import (
    GlobusOIDCIdPCheck,
    GO01,
    GO02,
    GO03,
    GO04,
    GO05,
    GO06,
    GO07,
    GO08,
    GO09,
    GO10,
    GO11,
    GO12,
    GO13,
    GO14,
    GO15,
    GO16,
    GO17,
    GO18,
    GO19,
    GO20,
)


def test_check_go01(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go01 = GO01()

    assert go01.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO01), res)) == []

    assert go01.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO01), res)) == []

    globus_oidc_idp.display_name = None
    assert not go01.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO01), res)) != []


def test_check_go02(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go02 = GO02()

    assert go02.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO02), res)) == []

    assert go02.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO02), res)) == []

    globus_oidc_idp.display_name = "\r\n"
    assert not go02.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO02), res)) != []


def test_check_go03(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go03 = GO03()

    assert go03.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO03), res)) == []

    assert go03.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO03), res)) == []

    globus_oidc_idp.support_email = None
    assert not go03.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO03), res)) != []


def test_check_go04(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go04 = GO04()

    assert go04.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO04), res)) == []

    assert go04.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO04), res)) == []

    globus_oidc_idp.support_email = "\r\n"
    assert not go04.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO04), res)) != []


def test_check_go05(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go05 = GO05()

    assert go05.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO05), res)) == []

    assert go05.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO05), res)) == []

    globus_oidc_idp.support_contact = None
    assert not go05.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO05), res)) != []


def test_check_go06(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go06 = GO06()

    assert go06.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO06), res)) == []

    assert go06.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO06), res)) == []

    globus_oidc_idp.support_contact = "\r\n"
    assert not go06.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO06), res)) != []


def test_check_go07(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go07 = GO07()

    assert go07.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO07), res)) == []

    assert go07.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO07), res)) == []

    globus_oidc_idp.server_name = None
    assert not go07.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO07), res)) != []


def test_check_go08(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go08 = GO08()

    assert go08.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO08), res)) == []

    assert go08.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO08), res)) == []

    globus_oidc_idp.server_name = "\r\n"
    assert not go08.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO08), res)) != []


def test_check_go09(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go09 = GO09()

    assert go09.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO09), res)) == []

    assert go09.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO09), res)) == []

    globus_oidc_idp.certificate_path = None
    assert not go09.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO09), res)) != []


def test_check_go10(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go10 = GO10()

    assert go10.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO10), res)) == []

    assert go10.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO10), res)) == []

    globus_oidc_idp.certificate_path = "\r\n"
    assert not go10.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO10), res)) != []


def test_check_go11(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go11 = GO11()

    assert go11.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO11), res)) == []

    assert go11.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO11), res)) == []

    globus_oidc_idp.certificate_chain_path = None
    assert not go11.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO11), res)) != []


def test_check_go12(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go12 = GO12()

    assert go12.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO12), res)) == []

    assert go12.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO12), res)) == []

    globus_oidc_idp.certificate_chain_path = "\r\n"
    assert not go12.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO12), res)) != []


def test_check_go13(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go13 = GO13()

    assert go13.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO13), res)) == []

    assert go13.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO13), res)) == []

    globus_oidc_idp.private_key_path = None
    assert not go13.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO13), res)) != []


def test_check_go14(globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go14 = GO14()

    assert go14.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO14), res)) == []

    assert go14.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO14), res)) == []

    globus_oidc_idp.private_key_path = "\r\n"
    assert not go14.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO14), res)) != []


def test_check_go15(tmp_path: pathlib.Path, globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go15 = GO15()

    existing = tmp_path / "existing.pem"
    with open(existing, "w") as f:
        print("---cert----", file=f)

    globus_oidc_idp.certificate_path = str(existing)
    assert go15.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO15), res)) == []

    assert go15.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO15), res)) == []

    globus_oidc_idp.certificate_path = str(tmp_path / "non-existing.pem")
    assert not go15.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO15), res)) != []


def test_check_go16(tmp_path: pathlib.Path, globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go16 = GO16()

    existing = tmp_path / "existing.pem"
    with open(existing, "w") as f:
        print("---cert----", file=f)

    globus_oidc_idp.certificate_chain_path = str(existing)
    assert go16.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO16), res)) == []

    assert go16.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO16), res)) == []

    globus_oidc_idp.certificate_chain_path = str(tmp_path / "non-existing.pem")
    assert not go16.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO16), res)) != []


def test_check_go17(tmp_path: pathlib.Path, globus_oidc_idp: GlobusOIDCIdP) -> None:
    checker = GlobusOIDCIdPCheck()
    go17 = GO17()

    existing = tmp_path / "existing.pem"
    with open(existing, "w") as f:
        print("---cert----", file=f)

    globus_oidc_idp.private_key_path = str(existing)
    assert go17.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO17), res)) == []

    assert go17.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO17), res)) == []

    globus_oidc_idp.private_key_path = str(tmp_path / "non-existing.pem")
    assert not go17.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)


def test_check_go18(
    tmp_path: pathlib.Path,
    test_host_cert_file: "os.PathLike[str]",
    globus_oidc_idp: GlobusOIDCIdP,
) -> None:
    checker = GlobusOIDCIdPCheck()
    go18 = GO18()

    globus_oidc_idp.certificate_path = str(test_host_cert_file)
    assert go18.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO18), res)) == []

    assert go18.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO18), res)) == []

    globus_oidc_idp.certificate_path = str(tmp_path / "existing.pem")
    with open(globus_oidc_idp.certificate_path, "w") as f:
        print("----", file=f)

    assert not go18.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO18), res)) != []


def test_check_go19(
    tmp_path: pathlib.Path,
    test_private_key_file: "os.PathLike[str]",
    globus_oidc_idp: GlobusOIDCIdP,
) -> None:
    checker = GlobusOIDCIdPCheck()
    go19 = GO19()

    globus_oidc_idp.private_key_path = str(test_private_key_file)
    assert go19.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO19), res)) == []

    assert go19.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO19), res)) == []

    globus_oidc_idp.private_key_path = str(tmp_path / "bad_key.pem")
    with open(globus_oidc_idp.private_key_path, "w") as f:
        print("----", file=f)

    assert not go19.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO19), res)) != []


def test_check_go20(
    tmp_path: pathlib.Path,
    test_host_cert_file: "os.PathLike[str]",
    globus_oidc_idp: GlobusOIDCIdP,
) -> None:
    checker = GlobusOIDCIdPCheck()
    go20 = GO20()

    globus_oidc_idp.certificate_chain_path = str(test_host_cert_file)
    assert go20.is_ok(None)
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, GO20), res)) == []

    assert go20.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO20), res)) == []

    globus_oidc_idp.certificate_chain_path = str(tmp_path / "existing.pem")
    with open(globus_oidc_idp.certificate_chain_path, "w") as f:
        print("----", file=f)

    assert not go20.is_ok(globus_oidc_idp)

    res = checker.check(globus_oidc_idp)
    assert list(filter(lambda x: isinstance(x, GO20), res)) != []
