from gcs_migration.typing.auth_documents import IdentityDocumentDict
from gcs_migration.models import HostEndpoint

from gcs_migration.check.host_endpoint import (
    HostEndpointCheck,
    EP01,
    EP02,
    EP03,
    EP04,
    EP05,
)


def test_check_ep01(host_endpoint: HostEndpoint, mock_get_auth_api: None) -> None:
    ep01 = EP01()

    assert not ep01.is_ok(None)
    assert ep01.is_ok(host_endpoint)

    checker = HostEndpointCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, EP01), res)) != []

    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP01), res)) == []


def test_check_ep02(host_endpoint: HostEndpoint, mock_get_auth_api: None) -> None:
    ep02 = EP02()
    host_endpoint.organization = None

    assert ep02.is_ok(None)
    assert not ep02.is_ok(host_endpoint)

    checker = HostEndpointCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, EP02), res)) == []

    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP02), res)) != []

    host_endpoint.organization = "org"
    assert ep02.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP02), res)) == []


def test_check_ep03(
    host_endpoint: HostEndpoint,
    identity_document: IdentityDocumentDict,
    mock_get_auth_api: None,
) -> None:
    ep03 = EP03()
    host_endpoint.owner = None

    assert ep03.is_ok(None)
    assert not ep03.is_ok(host_endpoint)

    checker = HostEndpointCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, EP03), res)) == []

    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP03), res)) != []

    host_endpoint.owner = identity_document["id"]
    assert ep03.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP03), res)) == []


def test_check_ep04(
    host_endpoint: HostEndpoint,
    identity_document: IdentityDocumentDict,
    unknown_identity_id: str,
    mock_get_auth_api: None,
) -> None:
    ep04 = EP04()
    host_endpoint.owner = None

    assert ep04.is_ok(None)
    assert ep04.is_ok(host_endpoint)

    checker = HostEndpointCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, EP04), res)) == []

    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP04), res)) == []

    host_endpoint.owner = identity_document["id"]
    assert ep04.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP04), res)) == []

    host_endpoint.owner = unknown_identity_id
    assert not ep04.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP04), res)) != []


def test_check_ep05(
    host_endpoint: HostEndpoint,
    identity_document: IdentityDocumentDict,
    unknown_identity_id: str,
    email_idp_identity_document: IdentityDocumentDict,
    mock_get_auth_api: None,
) -> None:
    ep05 = EP05()
    host_endpoint.owner = None

    assert ep05.is_ok(None)
    assert ep05.is_ok(host_endpoint)

    checker = HostEndpointCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, EP05), res)) == []

    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP05), res)) == []

    host_endpoint.owner = identity_document["id"]
    assert ep05.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP05), res)) == []

    host_endpoint.owner = unknown_identity_id
    assert ep05.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP05), res)) == []

    host_endpoint.owner = email_idp_identity_document["id"]
    assert not ep05.is_ok(host_endpoint)
    res = checker.check(host_endpoint)
    assert list(filter(lambda x: isinstance(x, EP05), res)) != []
