import pytest

from gcs_migration.models import MappedCollection

from gcs_migration.check.mapped_collection import (
    MappedCollectionCheck,
    MC01,
    MC02,
    MC03,
    MC04,
    MC05,
    MC06,
    MC07,
    MC08,
    MC09,
    MC10,
)


def test_check_mc01(mapped_collection: MappedCollection) -> None:
    mc01 = MC01()

    assert not mc01.is_ok(None)
    assert mc01.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC01), res)) != []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC01), res)) == []


def test_check_mc02(mapped_collection: MappedCollection) -> None:
    mc02 = MC02()

    assert mc02.is_ok(None)
    assert mc02.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC02), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC02), res)) == []

    mapped_collection.display_name = None

    assert not mc02.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC02), res)) != []


def test_check_mc03(mapped_collection: MappedCollection) -> None:
    mc03 = MC03()
    mapped_collection.force_verify = True

    assert mc03.is_ok(None)
    assert mc03.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    mapped_collection.force_verify = False
    assert mc03.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) == []

    mapped_collection.force_verify = None

    assert not mc03.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC03), res)) != []


def test_check_mc04(mapped_collection: MappedCollection) -> None:
    mc04 = MC04()
    mapped_collection.disable_verify = True

    assert mc04.is_ok(None)
    assert mc04.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    mapped_collection.disable_verify = False
    assert mc04.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) == []

    mapped_collection.disable_verify = None

    assert not mc04.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC04), res)) != []


def test_check_mc05(mapped_collection: MappedCollection) -> None:
    mc05 = MC05()
    mapped_collection.enable_https = True

    assert mc05.is_ok(None)
    assert mc05.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    mapped_collection.enable_https = False
    assert mc05.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) == []

    mapped_collection.enable_https = None

    assert not mc05.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC05), res)) != []


def test_check_mc06(mapped_collection: MappedCollection) -> None:
    mc06 = MC06()
    mapped_collection.allow_guest_collections = True

    assert mc06.is_ok(None)
    assert mc06.is_ok(mapped_collection)

    checker = MappedCollectionCheck()
    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    mapped_collection.allow_guest_collections = False
    assert mc06.is_ok(mapped_collection)
    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) == []

    mapped_collection.allow_guest_collections = None

    assert not mc06.is_ok(mapped_collection)

    res = checker.check(mapped_collection)
    assert list(filter(lambda x: isinstance(x, MC06), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,allow_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc07(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    allow_set: bool,
    expect: bool,
) -> None:
    mc07 = MC07()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if allow_set:
        mapped_collection.sharing_users_allow = ["user1"]
    else:
        mapped_collection.sharing_users_allow = None

    assert mc07.is_ok(None)
    assert mc07.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC07), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC07), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,deny_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc08(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    deny_set: bool,
    expect: bool,
) -> None:
    mc08 = MC08()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if deny_set:
        mapped_collection.sharing_users_deny = ["user1"]
    else:
        mapped_collection.sharing_users_deny = None

    assert mc08.is_ok(None)
    assert mc08.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC08), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC08), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,allow_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc09(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    allow_set: bool,
    expect: bool,
) -> None:
    mc09 = MC09()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if allow_set:
        mapped_collection.sharing_groups_allow = ["user1"]
    else:
        mapped_collection.sharing_groups_allow = None

    assert mc09.is_ok(None)
    assert mc09.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC09), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC09), res)) != []


@pytest.mark.parametrize(
    "allow_guest_collections,deny_set,expect",
    [
        (True, True, True),
        (True, False, True),
        (False, False, True),
        (False, True, False),
    ],
)
def test_check_mc10(
    mapped_collection: MappedCollection,
    allow_guest_collections: bool,
    deny_set: bool,
    expect: bool,
) -> None:
    mc10 = MC10()
    mapped_collection.allow_guest_collections = allow_guest_collections
    if deny_set:
        mapped_collection.sharing_groups_deny = ["user1"]
    else:
        mapped_collection.sharing_groups_deny = None

    assert mc10.is_ok(None)
    assert mc10.is_ok(mapped_collection) == expect

    checker = MappedCollectionCheck()
    res = checker.check(mapped_collection)
    if expect:
        assert list(filter(lambda x: isinstance(x, MC10), res)) == []
    else:
        assert list(filter(lambda x: isinstance(x, MC10), res)) != []
