from typing import Any, Generator, List, Optional, Tuple
from unittest import mock

import pytest
from click.testing import CliRunner

from gcs_migration.commands.check import check
from gcs_migration.models import MigrationPlan


@pytest.fixture
def mock_check_environment(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        yield migration_plan_cmd_create


@pytest.mark.parametrize(
    "args,expect",
    [
        ([], 7),
        (["--type-check", "acl"], 0),
        (["--type-check", "credential"], 2),
        (["--type-check", "endpoint"], 0),
        (["--type-check", "guest-collection"], 0),
        (["--type-check", "mapped-collection"], 3),
        (["--type-check", "migration-plan"], 0),
        (["--type-check", "oidc-server"], 0),
        (["--type-check", "role"], 0),
        (["--type-check", "storage-gateway"], 2),
        (["--type-check", "storage-gateway", "--type-check", "credential"], 4),
    ],
)
def test_check(
    mock_check_environment: MigrationPlan, args: List[str], expect: int,
) -> None:
    runner = CliRunner()
    result = runner.invoke(check, args)
    assert result.exit_code == expect
