import pytest

from gcs_migration.models import GlobusOIDCIdP, GlobusOIDCIdPDict


@pytest.fixture
def globus_oidc_display_name() -> str:
    return "Display Name"


@pytest.fixture
def globus_oidc_support_email() -> str:
    return "support@example.edu"


@pytest.fixture
def globus_oidc_support_contact() -> str:
    return "Support Admin Name"


@pytest.fixture
def globus_oidc_server_name() -> str:
    return "oidc.example.edu"


@pytest.fixture
def globus_oidc_certificate_path() -> str:
    return "/etc/grid-security/hostcert.pem"


@pytest.fixture
def globus_oidc_certificate_chain_path() -> str:
    return "/etc/grid-security/hostcert_chain.pem"


@pytest.fixture
def globus_oidc_private_key_path() -> str:
    return "/etc/grid-security/hostkey.pem"


@pytest.fixture
def globus_oidc_idp_dict(
    globus_oidc_display_name: str,
    globus_oidc_support_email: str,
    globus_oidc_support_contact: str,
    globus_oidc_server_name: str,
    globus_oidc_certificate_path: str,
    globus_oidc_certificate_chain_path: str,
    globus_oidc_private_key_path: str,
) -> GlobusOIDCIdPDict:
    return {
        "display_name": globus_oidc_display_name,
        "support_email": globus_oidc_support_email,
        "support_contact": globus_oidc_support_contact,
        "server_name": globus_oidc_server_name,
        "certificate_path": globus_oidc_certificate_path,
        "certificate_chain_path": globus_oidc_certificate_chain_path,
        "private_key_path": globus_oidc_private_key_path,
        "manage_certificate_and_key": True,
    }


@pytest.fixture
def globus_oidc_idp(
    globus_oidc_display_name: str,
    globus_oidc_support_email: str,
    globus_oidc_support_contact: str,
    globus_oidc_server_name: str,
    globus_oidc_certificate_path: str,
    globus_oidc_certificate_chain_path: str,
    globus_oidc_private_key_path: str,
) -> GlobusOIDCIdP:
    idp = GlobusOIDCIdP()

    idp.display_name = globus_oidc_display_name
    idp.support_email = globus_oidc_support_email
    idp.support_contact = globus_oidc_support_contact
    idp.server_name = globus_oidc_server_name
    idp.certificate_path = globus_oidc_certificate_path
    idp.certificate_chain_path = globus_oidc_certificate_chain_path
    idp.private_key_path = globus_oidc_private_key_path
    idp.manage_certificate_and_key = True

    return idp
