import pytest

from gcs_migration.models import Credential, CredentialDict, GuestCollection
from gcs_migration.typing.transfer_documents import (
    AccessListDocumentDict,
    AnonymousEffectivePermissionsDict,
    EndpointDocumentDict,
)


@pytest.fixture
def guest_collection(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential: CredentialDict,
) -> GuestCollection:
    guest_collection = GuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = Credential(credential)
    guest_collection.credential = cred

    return guest_collection


@pytest.fixture
def guest_collection_unresolved_credential(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential_unresolved: CredentialDict,
) -> GuestCollection:
    guest_collection = GuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = Credential(credential_unresolved)
    guest_collection.credential = cred

    return guest_collection
