import os
import pkg_resources
from typing import Generator, IO
from unittest import mock
import urllib.error

import pytest


@pytest.fixture
def incommon_urlopener() -> Generator[None, None, None]:
    def mocked_urlopen(url: str) -> IO:
        if url == "https://cilogon.org/include/idplist.xml":
            return pkg_resources.resource_stream(
                __name__, os.path.basename("idplist.xml")
            )
        elif url.startswith("https://mdq.incommon.org/entities/"):
            return pkg_resources.resource_stream(
                __name__, os.path.basename("InCommon-metadata.xml")
            )
        else:
            raise urllib.error.URLError(url)

    with mock.patch("gcs_migration.parsers.incommon_metadata.urlopen") as urlopen:
        urlopen.side_effect = mocked_urlopen

        yield
