from typing import Callable

import pytest

from gcs_migration.models import MappedCollection
from gcs_migration.parsers import GridFTPOptionParser
from gcs_migration.typing.transfer_documents import (
    EndpointDocumentDict,
    RoleListDocumentDict,
)

from tests.fixtures.gridftp_config import (
    GridFTPConfigFixture,
    GridFTPConfigFixtureGenerator,
    GridFTPConfigPolicy,
)


@pytest.fixture
def mapped_collection(
    host_endpoint_document: EndpointDocumentDict,
    gridftp_conf: GridFTPConfigPolicy,
    gridftp_config_d: GridFTPConfigFixtureGenerator,
    gridftp_config_gcs: GridFTPConfigPolicy,
    gridftp_config_authorization_myproxy: GridFTPConfigPolicy,
    gridftp_config_credential: GridFTPConfigPolicy,
    gridftp_config_logging: GridFTPConfigPolicy,
    gridftp_config_sharing_policies: GridFTPConfigPolicy,
    role_list_document: RoleListDocumentDict,
    fake_gridftp_config_parser_creator: Callable[
        [GridFTPConfigFixture], GridFTPOptionParser
    ],
) -> MappedCollection:
    configs = [
        gridftp_config_gcs,
        gridftp_config_authorization_myproxy,
        gridftp_config_credential,
        gridftp_config_logging,
        gridftp_config_sharing_policies,
    ]
    config_dir, config_dir_options = gridftp_config_d(configs)
    config_data = {**gridftp_conf.options, **config_dir_options}
    config_fixture = config_dir, config_data

    gridftp_config = fake_gridftp_config_parser_creator(config_fixture)

    mapped_collection = MappedCollection()
    mapped_collection.gridftp_config = gridftp_config
    mapped_collection.transfer_endpoint = host_endpoint_document
    mapped_collection.role_list_document = role_list_document

    return mapped_collection


@pytest.fixture
def mapped_collection_cmd_create(
    gridftp_config_parser: GridFTPOptionParser,
) -> MappedCollection:
    mapped_collection = MappedCollection()
    mapped_collection.gridftp_config = gridftp_config_parser

    return mapped_collection
