import pytest

from gcs_migration.models import (
    GlobusOIDCIdP,
    GuestCollection,
    HostEndpoint,
    MappedCollection,
    MigrationPlan,
    StorageGateway,
)
from gcs_migration.parsers import GCSConfig, GridFTPOptionParser

from .auth_idp import AuthIdP


@pytest.fixture
def migration_plan(
    host_endpoint: HostEndpoint,
    gcs_config: GCSConfig,
    gridftp_config_parser: GridFTPOptionParser,
    mapped_collection: MappedCollection,
    guest_collection_unresolved_credential: GuestCollection,
    globus_oidc_idp: GlobusOIDCIdP,
    storage_gateway: StorageGateway,
    auth_idp: AuthIdP,
) -> MigrationPlan:
    migration_plan = MigrationPlan()
    migration_plan.gcs_config = gcs_config
    migration_plan.gridftp_config = gridftp_config_parser
    migration_plan.set_host_endpoint(host_endpoint)
    if host_endpoint.subscription_id:
        mapped_collection.enable_https = True
    else:
        mapped_collection.enable_https = False
    migration_plan.mapped_collection = mapped_collection
    migration_plan.add_guest_collection(guest_collection_unresolved_credential)
    migration_plan.add_identity_provider(auth_idp)
    migration_plan.oidc_server = globus_oidc_idp
    migration_plan.storage_gateway = storage_gateway
    return migration_plan


@pytest.fixture
def migration_plan_cmd_create(
    host_endpoint: HostEndpoint,
    gcs_config: GCSConfig,
    gridftp_config_parser: GridFTPOptionParser,
    mapped_collection_cmd_create: MappedCollection,
    guest_collection_unresolved_credential: GuestCollection,
    storage_gateway_cmd_create: StorageGateway,
    auth_idp: AuthIdP,
) -> MigrationPlan:
    migration_plan = MigrationPlan()
    migration_plan.gcs_config = gcs_config
    migration_plan.gridftp_config = gridftp_config_parser
    migration_plan.set_host_endpoint(host_endpoint)
    if host_endpoint.subscription_id:
        mapped_collection_cmd_create.enable_https = True
    else:
        mapped_collection_cmd_create.enable_https = False
    migration_plan.mapped_collection = mapped_collection_cmd_create
    migration_plan.add_guest_collection(guest_collection_unresolved_credential)
    migration_plan.add_identity_provider(auth_idp)
    migration_plan.storage_gateway = storage_gateway_cmd_create
    return migration_plan
