import pytest

from gcs_migration.typing.transfer_documents import (
    AnonymousEffectivePermissionsDict,
    AccessDocumentDict,
    AccessListDocumentDict,
    EndpointDocumentDict,
    EndpointServerDocumentDict,
    RoleDocumentDict,
    RoleListDocumentDict,
    SharedEndpointListDict,
)


@pytest.fixture
def access_acl_document() -> AccessDocumentDict:
    return {
        "DATA_TYPE": "access",
        "id": 12345,
        "role_id": None,
        "role_type": None,
        "principal_type": "identity",
        "principal": "368e91db-2294-4b32-b344-6870afb3777d",
        "path": "/",
        "permissions": "r",
        "create_time": "2021-04-06T19:56:08+00:00",
    }


@pytest.fixture
def endpoint_server_document() -> EndpointServerDocumentDict:
    return {
        "DATA_TYPE": "server",
        "id": 985,
        "hostname": "gridftp.example.org",
        "port": 2811,
        "scheme": "gsiftp",
        "subject": "/O=Grid/OU=Example/CN=host/girdftp.example.org",
        "incoming_data_port_start": 50000,
        "incoming_data_port_end": 51000,
        "outgoing_data_port_start": 50000,
        "outgoing_data_port_end": 51000,
        "uri": "gsiftp://gridftp.example.org:2811",
        "is_connected": True,
        "is_paused": False,
    }


@pytest.fixture
def host_endpoint_uuid() -> str:
    return "096a65b1-3e90-44e6-9cb4-550b2d5401c0"


@pytest.fixture
def host_endpoint_document(
    endpoint_server_document: EndpointServerDocumentDict, host_endpoint_uuid: str,
) -> EndpointDocumentDict:
    return {
        "DATA_TYPE": "endpoint",
        "id": host_endpoint_uuid,
        "canonical_name": f"canonical#{host_endpoint_uuid}",
        "display_name": "myserver",
        "organization": "My Org",
        "department": "My Dept",
        "username": "auser",
        "description": "Example gridftp endpoint.",
        "public": False,
        "french_english_bilingual": False,
        "is_globus_connect": False,
        "globus_connect_setup_key": None,
        "gcp_connected": None,
        "gcp_paused": None,
        "gcs_version": None,
        "host_endpoint": None,
        "host_endpoint_display_name": None,
        "my_effective_roles": [],
        "network_use": "normal",
        "location": None,
        "local_user_info_available": None,
        "https_server": None,
        "high_assurance": False,
        "authentication_timeout_mins": None,
        "authentication_assurance_timeout": None,
        "non_functional": False,
        "max_concurrency": 4,
        "max_parallelism": 8,
        "preferred_concurrency": 2,
        "preferred_parallelism": 4,
        "s3_url": None,
        "s3_owner_activated": False,
        "host_endpoint_id": None,
        "host_path": None,
        "disable_verify": False,
        "force_verify": False,
        "force_encryption": False,
        "mfa_required": False,
        "myproxy_server": None,
        "myproxy_dn": None,
        "name": host_endpoint_uuid,
        "oauth_server": None,
        "default_directory": None,
        "activated": False,
        "expires_in": 0,
        "expire_time": "2000-01-02 03:45:06+00:00",
        "shareable": True,
        "acl_available": False,
        "acl_editable": False,
        "in_use": False,
        "contact_info": "Contact Info",
        "contact_email": "contact@globus.org",
        "info_link": "https://info.globus.org",
        "user_message": "Hello, users",
        "user_message_link": "https://globus.org",
        "keywords": "kw1,kw2",
        "subscription_id": "358d3618-553a-46d0-979c-9ed4ddc45c92",
        "owner_id": "64620d3a-f6d6-4e56-9b48-a73b3ac5f070",
        "owner_string": "owner@globus.org",
        "DATA": [endpoint_server_document],
    }


@pytest.fixture
def share_endpoint_uuid() -> str:
    return "e8d07a55-0e83-4b8f-a690-46371b1ae8a6"


@pytest.fixture
def shared_endpoint_document(
    share_endpoint_uuid: str, endpoint_server_document: EndpointServerDocumentDict,
) -> EndpointDocumentDict:
    return {
        "DATA_TYPE": "endpoint",
        "acl_available": True,
        "acl_editable": True,
        "activated": False,
        "authentication_assurance_timeout": None,
        "authentication_timeout_mins": None,
        "canonical_name": "u_urgkh35ixffxhks7krxnlk36my#02f5bb52-8109-11eb-9eb0-fdd0f285edc1",
        "contact_email": None,
        "contact_info": None,
        "default_directory": "/~/",
        "department": None,
        "description": None,
        "disable_verify": False,
        "display_name": "gridftp server",
        "expire_time": None,
        "expires_in": -1,
        "force_encryption": False,
        "force_verify": False,
        "french_english_bilingual": False,
        "gcp_connected": None,
        "gcp_paused": None,
        "gcs_version": None,
        "globus_connect_setup_key": None,
        "high_assurance": False,
        "host_endpoint": "u_urgkh35ixffxhks7krxnlk36my#a44ca3ef-a8b9-4b73-aa5f-546ed5ab7e66",
        "host_endpoint_display_name": "myserver",
        "host_endpoint_id": "096a65b1-3e90-44e6-9cb4-550b2d5401c0",
        "host_path": "/",
        "https_server": None,
        "id": share_endpoint_uuid,
        "in_use": False,
        "info_link": None,
        "is_globus_connect": False,
        "keywords": "kw",
        "local_user_info_available": None,
        "location": None,
        "max_concurrency": None,
        "max_parallelism": None,
        "mfa_required": False,
        "my_effective_roles": [
            "administrator",
            "access_manager",
            "activity_manager",
            "activity_monitor",
        ],
        "myproxy_dn": None,
        "myproxy_server": "myproxy.globusonline.org",
        "name": "02f5bb52-8109-11eb-9eb0-fdd0f285edc1",
        "network_use": None,
        "non_functional": False,
        "oauth_server": None,
        "organization": None,
        "owner_id": "64620d3a-f6d6-4e56-9b48-a73b3ac5f070",
        "owner_string": "owner@globus.org",
        "preferred_concurrency": None,
        "preferred_parallelism": None,
        "public": True,
        "s3_owner_activated": False,
        "s3_url": None,
        "shareable": False,
        "subscription_id": None,
        "user_message": "foo",
        "user_message_link": None,
        "username": "auser",
        "DATA": [],
    }


@pytest.fixture
def access_role_document() -> AccessDocumentDict:
    return {
        "DATA_TYPE": "access",
        "id": None,
        "principal_type": "identity",
        "principal": "ce5a2f3a-9aa0-4d8b-a062-63c61878a10d",
        "role_type": "access_manager",
        "role_id": "87973c89-cd42-40d6-9dae-74bbb2aa6699",
        "path": None,
        "permissions": None,
        "create_time": None,
    }


@pytest.fixture
def role_document() -> RoleDocumentDict:
    return {
        "DATA_TYPE": "role",
        "id": "1405823f-0597-4a16-b296-46d4f0ae4b15",
        "principal_type": "identity",
        "principal": "ce5a2f3a-9aa0-4d8b-a062-63c61878a10d",
        "role": "access_manager",
    }


@pytest.fixture
def role_list_document(role_document: RoleDocumentDict) -> RoleListDocumentDict:
    return {"DATA_TYPE": "role_list", "DATA": [role_document]}


@pytest.fixture
def anonymous_effective_permissions_document(
    test_user_cert_data: str, sharing_uuid: str
) -> AnonymousEffectivePermissionsDict:
    return {
        "DATA_TYPE": "effective_permissions",
        "user_cert_chain": test_user_cert_data,
        "sharing_uuid": sharing_uuid,
        "DATA": [],
    }


@pytest.fixture
def access_list_document(
    access_acl_document: AccessDocumentDict, access_role_document: AccessDocumentDict
) -> AccessListDocumentDict:
    return {
        "length": 2,
        "endpoint": "u_urgkh35ixffxhks7krxnlk36my#02f5bb52-8109-11eb-9eb0-fdd0f285edc1",
        "DATA": [access_acl_document, access_role_document],
        "DATA_TYPE": "access_list",
    }


@pytest.fixture
def shared_endpoint_list_document(share_endpoint_uuid: str) -> SharedEndpointListDict:
    return {"next_token": None, "shared_endpoints": [{"id": share_endpoint_uuid}]}
