from typing import List

import pytest

from gcs_migration.parsers import parse_restrict_paths


@pytest.mark.parametrize("read", [[], ["~", "$HOME/a", "/a", "/b"]])
@pytest.mark.parametrize("read_write", [[], ["/d", "/e"]])
@pytest.mark.parametrize("none", [[], ["/d", "/e"]])
def test_parse_restrict_paths(
    read: List[str], read_write: List[str], none: List[str]
) -> None:
    restrict_paths_strings: List[str] = []
    for path in read:
        restrict_paths_strings.append(f"R{path}")
    for path in read_write:
        restrict_paths_strings.append(f"RW{path}")
    for path in none:
        restrict_paths_strings.append(f"N{path}")

    if restrict_paths_strings:
        restrict_path_string = ",".join(restrict_paths_strings)
        expected = {
            "DATA_TYPE": "path_restrictions#1.0.0",
            "read": read,
            "read_write": read_write,
            "none": none,
        }

        res = parse_restrict_paths(restrict_path_string)
        assert expected == res
